/*

by Luigi Auriemma

*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/stat.h>
#include "q3huff.h"
#include "show_dump.h"



#define VER     "0.2"
#define MOLT    4



void std_err(void);



int main(int argc, char *argv[]) {
    FILE            *fd;
    unsigned char   *buff,
                    mode;
    int             len,
                    off = 12;
    struct  stat    xstat;


    setbuf(stdout, NULL);

    fputs("\n"
        "Q3huffdecenc "VER"\n"
        "by Luigi Auriemma\n"
        "e-mail: aluigi@autistici.org\n"
        "web:    aluigi.org\n"
        "\n", stdout);

    if(argc < 3) {
        printf("\nUsage: %s <mode> <file_Q3_dump> [output_file]\n"
            "\n"
            "Mode:\n"
            "d = decoding/decompression\n"
            "e = encoding/compression\n"
            "\n"
            "Note: default output is hex visualization to stdout\n"
            "\n", argv[0]);
        exit(1);
    }

    mode = argv[1][0];

        /* opening file with the packet */

    fd = fopen(argv[2], "rb");
    if(!fd) std_err();
    if(fstat(fileno(fd), &xstat)
      < 0) std_err();
    buff = malloc(xstat.st_size);
    if(!buff) std_err();
    fread(buff, xstat.st_size, 1, fd);
    fclose(fd);

    printf("Your dump is %u bytes long\n", xstat.st_size);

    len = xstat.st_size * MOLT;    // decoding creates a bigger packet
    buff = realloc(buff, len);
    if(!buff) std_err();
    printf("Maximum decoded/encoded length supported is %d times the file size,\n"
           "so max %d bytes (the bytes in excess will be dropped)\n",
           MOLT, len);

        /* decoding/decompression and encoding/compression */

    if(buff[4] == 0x02) off = 13;

    switch(mode) {
        case 'd': {
            len = Huff_DecompressPacket(buff, off, xstat.st_size, len);
            } break;
        case 'e': {
            len = Huff_CompressPacket(buff, off, xstat.st_size);
            } break;
        default: {
            printf("\nWrong mode selected (%c), choose d or e\n", mode);
            exit(1);
        }
    }

    printf("Now the length is: %d\n\n", len);

    if(argc > 3) {
        fd = fopen(argv[3], "wb");
        if(!fd) std_err();
        fwrite(buff, len, 1, fd);
        fclose(fd);
    } else {
        show_dump(buff, len, stdout);
    }

    free(buff);
    return(0);
}



void std_err(void) {
    perror("\nError");
    exit(1);
}


